%
%  Solution to vibrational modes of CO2 Problem
%
%  David Keffer
%  Department of Chemical Engineering
%  University of Tennessee
%  Knoxville Tennessee
%  September, 2001
%
clear all;
close all;
%
tol = 1.0e-14;
% input
   mass(1) = 16;
   mass(2) = 12;
   mass(3) = 16;
   k12 = 48;
   k13 = 48;
   A = [ 	0					0							0					1					0					0
      		0					0							0					0					1					0
				0					0							0					0					0					1
            -k12/mass(1)	k12/mass(1)				0					0					0					0
            k12/mass(2)		(-k12-k13)/mass(2)   k13/mass(2)		0					0					0
            0					k13/mass(3)				-k13/mass(3)	0					0					0];
n = max(size(A));
yo = [1.0;  0.0; -0.5; 0.0; 0.0; 0.0];
% constant terms in ODE
b = zeros(n,1);
to = 0.0;
tf = 10.0;
% eigenvalues and eigenvectors
[wcol,lambdac] = eig(A)
wcolinv = inv(wcol);
% calculate exp (-lambda*to) matrix
explambdano = zeros(n,n);
for j = 1:1:n
	explambdano(j,j) = exp(-lambdac(j,j)*to);
end
% calculate u(to)
uo = zeros(n,n);
for j = 1:1:n
   if (abs(lambdac(j,j)) > tol)
      uo(j,j) = -explambdano(j,j)/lambdac(j,j);
   else
      uo(j,j) = to;
   end
end

% set up discretized solution grid
nintervals = 1000;
npoints = nintervals + 1;
dt = (tf-to)/nintervals;
for i = 1:1:npoints
   tp(i) = (i-1)*dt +to;
end

% calculate solution
yp = zeros(n,npoints);
for i = 1:1:npoints
   explambdan = zeros(n,n);
   explambda = zeros(n,n);
	for j = 1:1:n
		explambdan(j,j) = exp(-lambdac(j,j)*tp(i));
		explambda(j,j) = exp(lambdac(j,j)*tp(i));
	end
	% calculate u(t)
	u = zeros(n,n);
	for j = 1:1:n
   	if (abs(lambdac(j,j)) > tol)
      	u(j,j) = -explambdan(j,j)/lambdac(j,j);
   	else
      	u(j,j) = tp(i);
   	end
   end
   term1 = explambdano*wcolinv*yo;
   term2 = (u-uo)*wcolinv*b;
   yp(:,i) = wcol*explambda*(term1 + term2);
end
% remove residual imaginary components due to round-off
for i =1:1:npoints
   for j = 1:1:n
      if (imag(yp(j,i)) < tol)
         yp(j,i) = real( yp(j,i) );
      end
   end
end
      
      
      
%plot
for j = 1:1:n 
  if (j==1)
     plot (tp,yp(j,:),'g-')
  elseif (j==2)
     plot (tp,yp(j,:),'r-')
  elseif (j==3)
     plot (tp,yp(j,:),'b-')
  elseif (j==4)
     plot (tp,yp(j,:),'g:')
  elseif (j==5)
     plot (tp,yp(j,:),'r:')
  elseif (j==6)
     plot (tp,yp(j,:),'b:')
  end
  hold on
end
hold off
xlabel('time ');
ylabel('position  & velocity ');
legend('pos O1','pos C', 'pos O2','vel O1','vel C', 'vel O2');

yp(:,npoints)



