%
% Solution to the Continuous Stirred Tank Reactor Problem
% Systemn of First Order Linear Ordinary Differential Equations 
% Non-Homogeneous
% Eigenvalues can be zero
%
% David Keffer
% Department of Chemical Engineering
% University of Tennessee
% Knoxville Tennessee
% September, 2001
%
tol = 1.0e-14;
% input
k12 = 0.50;
k21 = 0.25;
k13 = 0.20;
k31 = 0.05;
k23 = 0.30;
k32 = 0.15;
A = [ (-k13-k12), k21, k31; k12, (-k21-k23), k32; k13, k23, (-k31-k32)];
n = max(size(A));
yo = [1.0/3.0; 1.0/3.0; 1.0/3.0];
%yo = [1.0; 0.0; 0.0];
% constant terms in ODE
b = [-0.01; -0.01; 0.02];
to = 0.0;
tf = 20.0;
% eigenvalues and eigenvectors
[wcol,lambdac] = eig(A);
wcolinv = inv(wcol);
% calculate exp (-lambda*to) matrix
explambdano = zeros(n,n);
for j = 1:1:n
   explambdano(j,j) = exp(-lambdac(j,j)*to);
end
% calculate u(to)
uo = zeros(n,n);
for j = 1:1:n
   if (abs(lambdac(j,j)) > tol)
      uo(j,j) = -explambdano(j,j)/lambdac(j,j);
   else
      uo(j,j) = to;
   end
end
% set up discretized solution grid
nintervals = 1000;
npoints = nintervals + 1;
dt = (tf-to)/nintervals;
for i = 1:1:npoints
   tp(i) = (i-1)*dt +to;
end
% calculate solution
yp = zeros(n,npoints);
for i = 1:1:npoints
   explambdan = zeros(n,n);
   explambda = zeros(n,n);
for j = 1:1:n 
   explambdan(j,j) = exp(-lambdac(j,j)*tp(i));
   explambda(j,j) = exp(lambdac(j,j)*tp(i));
end
% calculate u(t)
u = zeros(n,n);
for j = 1:1:n
   if (abs(lambdac(j,j)) > tol)
      u(j,j) = -explambdan(j,j)/lambdac(j,j);
   else
      u(j,j) = tp(i);
   end
end
term1 = explambdano*wcolinv*yo;
term2 = (u-uo)*wcolinv*b;
yp(:,i) = wcol*explambda*(term1 + term2);
end
%plot
for j = 1:1:n
   if (j==1)
      plot (tp,yp(j,:),'g-'), xlabel( 't' ), ylabel ( 'y' )
   elseif (j==2)
      plot (tp,yp(j,:),'r-'), xlabel( 't' ), ylabel ( 'y' )
   elseif (j==3)
      plot (tp,yp(j,:),'b-'), xlabel( 't' ), ylabel ( 'y' )
   end
   hold on
end
hold off
xlabel('time (s)');
ylabel('mole fraction');
legend('1 ','2 ', '3 ');