function [x,y]=euler1(n,xo,xf,yo);
%
%  [x,y]=euler1(n,xo,xf,yo);
%
% This script will solve one first-order ODE using the Euler method.
%
% ODE:  dy/dx = f(x,y)
%
% IC:   y(xo) = yo
%
% inputs:
% n = number of intervals, scalar
% x0 = value of x at initial condition, scalar
% xf = final value of x, scalar
% y0 = value of y at initial condition, scalar
% f(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = vector of x values, nx1 vector
% y = vector of y values, nx1 vector
% a graph of y vs x
% a file, 'euler1_out.txt' containing tabulated data of x and y
%
% example usage:
%
%  [x,y]=euler1(10,0,2,1);
%
% Author:  David Keffer 
% original date: October 23, 1998
% last modified date: April 2, 2013
%

%
% define discretization and x vector
%
dx = (xf-xo)/n;
x = zeros(n+1,1);
for i = 1:1:n+1
   x(i) = xo + (i-1)*dx;
end
%
% perform Euler steps
%
y = zeros(n+1,1);
y(1) = yo;
for i =  1:1:n
   dydx = funkeval(x(i),y(i));
      y(i+1) = y(i) + dx*dydx;
end
%
%  plot
%
close all;
iplot = 1;
if (iplot == 1)
   plot (x,y,'k-o'), xlabel( 'x' ), ylabel ( 'y' );
end
%
%  write result to file 'euler1_out.txt'
%
fid = fopen('euler1_out.txt','w');
fprintf(fid,'x                y \n');
fprintf(fid,'%23.15e %23.15e   \n', [x,y]');
fclose(fid);

%
% enter ODE in this function
%
function dydx = funkeval(x,y);
dydx = -1.0*y;
